#include <htc.h>	// Required to interface with delay routines
#include<math.h>  // used for ph calc
#include "header.h"
#ifndef _XTAL_FREQ
 #define _XTAL_FREQ 20000000
#endif

void fnADCInit(char vref,char channel)
{
	// disable comparator
	CMCON=7; // This apparently disables comparators
	// Set PORTA as input
	TRISA=0xff;

	TRISA4=0;		// Set Pt100's gnd pin as output
	
	// the passed parameter "vref" denotes the low/high voltage refrences
	// For example, "5" means - 0 V and 5 V	
	if(vref==3)
	{
		PCFG0=1;
		PCFG1=0;
		PCFG2=0;
		PCFG3=0;
		
	}
	else if(vref==5)
	{
		PCFG0=0;
		PCFG1=0;
		PCFG2=0;
		PCFG3=0;
	}
	else if(vref==13)
	{
		PCFG0=0;
		PCFG1=0;
		PCFG2=0;
		PCFG3=1;
	}
	
	// now that VRefs have been set, do the rest of the ADC setup
	// ALL READINGS USE THIS CONFIG --
	// * SLEEP MODE
	// * 10 BIT
	// * RIGHT ADJUST
	
	// setting clock to tosc/16
	ADCS2=0;		ADCS1=0;		ADCS0=1;			
		
	// Right Justified
		ADFM=1;
		
	// now select channel
	if(channel=='h')
	{
		CHS0=0;
		CHS1=0;
		CHS2=0;
	}
	else if(channel=='t')
	{
		CHS0=1;
		CHS1=0;
		CHS2=0;
	}	
	else if(channel=='1')
	{
		CHS0=0;
		CHS1=1;
		CHS2=0;
	}
	else if(channel=='3')
	{
		CHS0=1;
		CHS1=1;
		CHS2=0;
	}	
	else if(channel=='s')
	{
		CHS0=0;
		CHS1=0;
		CHS2=1;
	}
	
	else if(channel=='p')
	{
		// This is for the pot
		CHS0=1;
		CHS1=1;
		CHS2=1;
	}

	// Enable ADC interrupt
	//ADIE=1;
	//PEIE=1;
	//GIE=1;
	
	// THIS COMPLETES ADC SETUP !
}
	
	

int  fnADCConv1024Avg()
{
	// calib 3v Vref first !
	//fnCalibVRef();
	
	
	int i1024Result=0;
	for(char a=0;a<32;a++)
	{
		int iResult=0;
		cModeFlag=0; // clear flag
		
		for(char c=0;c<32;c++)
		{	
			// Turn ADC on
			ADON=1;
			// Wait for acquisition
			__delay_us(10);
			
			T0IE = 0;			// disable timer interrupt
			// START !
			GO_nDONE=1;	
			while(GO_nDONE);
			
			T0IE = 1;			// enable timer interrupt
			// averaging 32 times.So max value of iResult (int!) remains <65k
			iResult=iResult+((int)(ADRESH<<8)|(int)(ADRESL));
			
		}
		
		// The blow needed because the mode button does not respond
		// while the ADC is making a string of conversions (1024 or above)
		// and I forgot to wire up the mode button as an
		// interrupt. So we need to poll mode each time the 
		// adc conversion function is called.This makes sure user's
		// wishes don't go unheeded :)		
		// poll Mode
			if(RD3==0)
				{				
					if(fnDebounce(2)) // check debounce. "2" means "Mode.
					{	
						// check if we were at highest value
						if(cMode==3)
						{
							cMode=1;
						}
						else
						{
							cMode++;
						}
						
						// Now that mode has changed,set the flag
						cModeFlag=0;
						//Now exit.Further execution will be taken care of
						// by the *** CALLING FUNCTION ***
						return(0);
						
					}
				}
		
		
		iResult=(iResult>>5);
		i1024Result=i1024Result+iResult;
	}	
	i1024Result=(i1024Result>>5);
	return(i1024Result);	
}



int fnADCConv32Avg()
{
	
		int iResult=0;
		
		for(char c=0;c<32;c++)
		{	
			// Turn ADC on
			ADON=1;
			// Wait for acquisition
			__delay_us(10);			
			T0IE = 0;			// disable timer interrupt
			// START !
			GO_nDONE=1;	
			while(GO_nDONE);			
			T0IE = 1;			// enable timer interrupt
			// averaging 32 times.So max value of iResult (int!) remains <65k
			iResult=iResult+((int)(ADRESH<<8)|(int)(ADRESL));
		}
		
		return(iResult>>5);
		
		
}		
void fnCalib7pH()
{
	fnADCInit(5,'h');
	float fCompareReading2=0;
	
	// Just run adc conv. 4 times to pass time !!
	// also scroll dashes
	gcDashPos=0;
	for(char u=0;u<8;u++)
	{
		// scroll dashes !
		fnDashScroll();
		
		// take reading
		for(char k=0;k<8;k++)
		{	
			fnADC1024AvgUnbirdled();			
		}
		
		// Average of the 8 averages
		fCompareReading2=((int)fCompareReading2>>3);	
	}
	

	
	// about 4 seconds passed. Take the actual reading !
		// scroll dashes !
		fnDashScroll();	

	for(char k=0;k<32;k++)
		{	
			fCompareReading2=fCompareReading2+fnADC1024AvgUnbirdled();					
		}
		// reset the dash-scroll var
		gcDashPos=0;
	// Average of the 16 averages
	fCompareReading2=((int)fCompareReading2>>5);
		
	fCompareReading2=(((fCompareReading2/1023)*(5000)))/5;
	
	gf7pHOffset=fCompareReading2 - 500;
	// Cal done !!	
}	


char fnCalib4pH()
{
	
	// But this point (unlike 7 pH) is not isothermal
	// So we need to know the temperature to calibrate
	int	iTempC=30;
	
	
	// This var is used to denote the mode being used !
	char cOperatingMode;
	
	

			cOperatingMode='M';	
		
			cMode=1;		// 1 for manual	

	
	
	fnADCInit(5,'h');
	float fCompareReading2=0;
	
	//************************************************
	// Just run adc conv. a few times to pass time !!
	// also scroll dashes
	//************************************************
	gcDashPos=0;
	for(char u=0;u<8;u++)
	{
			// scroll dashes !
			fnDashScroll();
		for(char k=0;k<8;k++)
		{	
			fnADC1024AvgUnbirdled();			
		}
	}	
	
	
	// Some time passed. Take the actual reading !
			// scroll dashes too !
			fnDashScroll();
	for(char k=0;k<32;k++)
		{	
			fCompareReading2=fCompareReading2+fnADC1024AvgUnbirdled();			
		}
		// reset the dash-scroll var
		gcDashPos=0;
		
		// Average of the 16 averages
		fCompareReading2=((int)fCompareReading2>>5);
	
	fCompareReading2=(((fCompareReading2/1023)*(5000)))/5;
	fCompareReading2= fCompareReading2 - gf7pHOffset - 500;
	
	// we now have the reading in mV.
	// Go for the temperature.
	// First check the mode. If ATC,GO FO PT100
	// If manual,just take the knob's value !
	

		fnADCInit(5,'p');	
		iTempC=((fnADCConv32Avg()/10));



	
	// Now set the pH slope	
	fnSetpHSlope(iTempC);
	
	// Now that we the compensated slope,let's calc the offset
	
	gfOffsetPermV=((pHSlope*3)-fCompareReading2) ;
	gfOffsetPermV= gfOffsetPermV / (fCompareReading2);
/*
				
	
	// DEBUG !!!!!
	fnDispNum(fCompareReading2);
	fnDelayTwoSecond();
	fnDispNum(iTempC);
	fnDelayTwoSecond();
	fnDispNum(gfOffsetPermV*1000);
	fnDelayTwoSecond();
	fnDispNum(pHSlope);
	fnDelayTwoSecond();
*/
		// Cal done !!	
		return(0); // 0 ==  sucess !

}

int fnCalcTempPt100()
{	
	// Open the Pt100's gnd !
	RA4=0;
	__delay_us(50);  // To allow for the track capacitances

	fnADCInit(13,'t');
		
	int iTempRead=fnADCConv1024Avg();
	if(cModeFlag==1)
		{
			// Dont reset flag as this func is called from the 
			// ATC func and we ned to return from there too !
			return;
		}
		
	// Read gnd potential
	fnADCInit(3,'s');

	float fGndReadmV=(fnADCConv1024Avg()*2.94);  // This gives the mV @ ground
	
	if(cModeFlag==1)
		{
			// Dont reset flag as this func is called from the 
			// ATC func and we ned to return from there too !
			return;
		}
	
	
		
	float fFrac=0;
	
	// mvReading / (5000-ground voltage) = frac
	fFrac=(((((float)iTempRead*2500)/1023)+500) - fGndReadmV )/ (5000-fGndReadmV) ; 
	
	
	// calc temp
	
	// subtract 100 as it's the 0deg value of pT100
	int iTempC=round((pT100Rcoeff)*((pT100Rvalue*(fFrac/(1-fFrac))) - 100 ));  //pT100Rcoeff
	
	// turn off the gnd
	RA4=1;
	
	return(iTempC); // !!!
}	


void fnSetpHSlope(int iTemp)
{
	// At 30 deg C,we get 53.33mV/pH
	// With ^ in temp, this too ^
	// The rate is 0.2mV per deg C
	pHSlope = 53.33 + ((iTemp-30) * 0.2);
	
}	

void fnCalibVRef()
{
	// Ideally,we should calib both,1v and 3v Vrefs
	// But,as I've found out,1v has significant noise on it
	// (because it's connected to pH sensor)
	// So it's noise and the sensor's noise cancel out !
	// Hence we calib only 3v
	
	// 3v
	//fnADCInit(5,'3');
	
	// 3%5=614 counts.Accordingly,calc offset
	// This value is in mV
	
	//gfVrefOffset = (614- fnADCConv1024Avg())*5;
	
	return;		
}	

int fnADC1024AvgUnbirdled()
{
	
	int i1024Result=0;
								for(char a=0;a<32;a++)
								{
									int iResult=0;
									cModeFlag=0; // clear flag
									
									for(char c=0;c<32;c++)
									{	
										// Turn ADC on
										ADON=1;
										// Wait for acquisition
										__delay_us(10);
										T0IE = 0;			// disable timer interrupt
										// START !
										GO_nDONE=1;	
										while(GO_nDONE);					
										T0IE = 1;			// enable timer interrupt
										// averaging 32 times.So max value of iResult (int!) remains <65k
										iResult=iResult+((int)(ADRESH<<8)|(int)(ADRESL));
									}
									
									iResult=(iResult>>5);
									i1024Result=i1024Result+iResult;
								}	
								i1024Result=(i1024Result>>5);
		return (i1024Result);
}

	